@@ -79,7 +79,9 @@ module POSIX
   # When a hash is given in the last argument (options), it specifies a
   # current directory and zero or more fd redirects for the child process.
   #
-  # The :chdir option specifies the current directory:
+  # The :chdir option specifies the current directory. Note that :chdir is not
+  # thread-safe on systems that provide posix_spawn(2), because it forces a
+  # temporary change of the working directory of the calling process.
   #
   #     spawn(command, :chdir => "/var/tmp")
   #
@@ -210,6 +212,8 @@ module POSIX
               if fd?(val)
                 val = fd_to_io(val)
                 key.reopen(val)
+                key.close_on_exec = false
+                val.close_on_exec = false
               elsif val == :close
                 if key.respond_to?(:close_on_exec=)
                   key.close_on_exec = true
@@ -236,7 +240,7 @@ module POSIX
           Process::setpgid(0, pgroup) if pgroup
 
           # do the deed
-          ::Kernel::exec(*argv)
+          ::Kernel::exec(*argv, :close_others=>false)
         ensure
           exit!(127)
         end
@@ -267,12 +271,7 @@ module POSIX
     # Returns the String output of the command.
     def `(cmd)
       r, w = IO.pipe
-      if RUBY_PLATFORM =~ /(mswin|mingw|cygwin|bccwin)/
-        sh = ENV['COMSPEC'] || 'cmd.exe'
-        pid = spawn([sh, sh], '/c', cmd, :out => w, r => :close)
-      else
-        pid = spawn(['/bin/sh', '/bin/sh'], '-c', cmd, :out => w, r => :close)
-      end
+      pid = spawn(*system_command_prefixes, cmd, :out => w, r => :close)
 
       if pid > 0
         w.close
@@ -488,6 +487,27 @@ module POSIX
       end
     end
 
+    # Derives the shell command to use when running the spawn.
+    #
+    # On a Windows machine, this will yield:
+    #   [['cmd.exe', 'cmd.exe'], '/c']
+    # Note: 'cmd.exe' is used if the COMSPEC environment variable
+    #   is not specified. If you would like to use something other
+    #   than 'cmd.exe', specify its path in ENV['COMSPEC']
+    #
+    # On all other systems, this will yield:
+    #   [['/bin/sh', '/bin/sh'], '-c']
+    #
+    # Returns a platform-specific [[<shell>, <shell>], <command-switch>] array.
+    def system_command_prefixes
+      if RUBY_PLATFORM =~ /(mswin|mingw|cygwin|bccwin)/
+        sh = ENV['COMSPEC'] || 'cmd.exe'
+        [[sh, sh], '/c']
+      else
+        [['/bin/sh', '/bin/sh'], '-c']
+      end
+    end
+
     # Converts the various supported command argument variations into a
     # standard argv suitable for use with exec. This includes detecting commands
     # to be run through the shell (single argument strings with spaces).
@@ -503,7 +523,7 @@ module POSIX
     def adjust_process_spawn_argv(args)
       if args.size == 1 && args[0] =~ /[ |>]/
         # single string with these characters means run it through the shell
-        [['/bin/sh', '/bin/sh'], '-c', args[0]]
+        [*system_command_prefixes, args[0]]
       elsif !args[0].respond_to?(:to_ary)
         # [argv0, argv1, ...]
         [[args[0], args[0]], *args[1..-1]]
